#include "config.h"
#include <sys/mount.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>

#include <badpenguin-ansi.h>
#include <badpenguin-chain.h>
#include <badpenguin-fs.h>
#include <badpenguin-hardware.h>

#include "caronte.h"
#include "caronte-logo.h"
#include "caronte-partitions.h"

TChain  *distro=NULL;
TPartition *sourcedist=NULL;

void caronte_distribution() {
  int iret;
  TChainItem *pitem=NULL;
  TPartition *ppart=NULL;

  // Print Distributions
  caronte_title(NULL);
  printf( "\n" );
  printf( "                        Ricerca Distribuzione\n\n" );
  printf( " Device      Versione                                  Tipo\n" );
  printf( " ==========  ========================================  ==================\n" );
  
  // Make Mount Point and Make sure it is unmounted
  umount(CARONTE_SOURCE_DIR);
  makedir(CARONTE_SOURCE_DIR, S_IRWXU);
  if (distro==NULL) {
    distro=chain_create();
  } else {
    chain_cleanall(distro);
  }
  
  // Mount all the partition from the list
  for ( pitem=part->head; pitem!=NULL; pitem=pitem->next) {
    ppart = TPARTITION(pitem->data);
    if ( 0 == strcmp(ppart->module, FILESYSTEM_NONE)) continue;
    if ( 0 == strcmp(ppart->module, FILESYSTEM_SWAP)) continue;
    iret = mount( 
      ppart->device, CARONTE_SOURCE_DIR, ppart->module, 
      MS_MGC_VAL || MS_RDONLY,
      NULL
    );
    // Errore nel mounting
    if (iret!=0) {
      printf(" %-10s  %-40s  %s\n", ppart->device, "Impossibile montare la partizione", ppart->filesystem );
      continue;
    }
    // Controllo presenza distribuzione
    if ( 0 == file_exists(CARONTE_SOURCE_DIST)) {
      //TODO: ricava versione della distribuzione
      printf(" %-10s  %-40s  %s\n", ppart->device, "Bad Penguin x.y.z", ppart->filesystem );
      pitem = chain_append(distro);
      pitem->name = "Bad Penguin x.y.z"; //TODO: nome distribuzione
      pitem->data = ppart;
    } else {
      printf(" %-10s  %-40s  %s\n", ppart->device, "Non contiene la distribuzione", ppart->filesystem );
    }      
    //iret = umount2(ppart->device, MNT_FORCE);
    iret = umount2(CARONTE_SOURCE_DIR, MNT_FORCE);
    if (iret!=0) {
      printERROR("             Errore nello smontare la partizione ! (%i,%i)\n",
        iret,
	errno
      );
      normal_color();
    }
  }

  // Check local distribution
  if ( 0 == file_exists(CARONTE_SOURCE_LOCAL)) {
    //TODO: ricava versione della distribuzione
    printf(" %-10s  %-40s  %s\n", "/", "Bad Penguin x.y.z", "ext2fs" );

    ppart = create_partition();
    ppart->device = strdup("/");
    ppart->type   = 0xFF;
    ppart->filesystem = strdup("Local filesystem");
    ppart->module     = strdup(FILESYSTEM_EXT2);
    ppart->bootflag   = 0;
    ppart->size       = 0;

    pitem = chain_append(distro);
    pitem->name = "Bad Penguin x.y.z"; //TODO: nome distribuzione
    pitem->data = ppart;
  }


  PRESS_ANY_KEY

}
