#include "config.h"
#include <sys/klog.h>
#include <sys/stat.h>
#include <sys/mount.h>
#include <sys/reboot.h>

#include <signal.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <badpenguin-ansi.h>
#include <badpenguin-fs.h>

#include "caronte.h"
#include "caronte-logo.h"


void caronte_reboot() {
  // Reboot
  printf("\nsincronizzio i dischi: ");
  sync(); sync();
  printf("OK\n");

  if (debug!=1) {
    printf("chiudo tutti i processi... ");
    kill(-1, 15);
    sleep(2);
    printf("fatto\n");

    printf("uccido eventuali processi rimasti...");
    kill(-1, 9);
    sleep(2);
    printf("fatto\n");
  }
  
  printf("smonto /proc: ");
  if (debug==1) {
    printf("Saltato\n");
  } else {
    umount("/proc");
    printf("OK\n");
  }
      
  printf("Riavvio del sistema in 4 secondi...\n");
  shutdown_flag=1;
  if (debug==1) {
    printf("Saltato\n");
    exit(1);
  } else {    
    sleep(4);
    printf("reboot!\n");
    sync();
    reboot(RB_AUTOBOOT);
  }
}



void caronte_signal(int sig) {
  color(CURSOR_ON);
  if (sig>0) {
    printf("\ncaronte: ricevuto il segnale %i\n", sig);
    if (shutdown_flag==1) return;
  } else {
    printf("caronte: completato!\n");
  }
  if (shutdown_flag==0) {
    caronte_reboot();
    printf("mmm... sono ancora vivo :-P\n");
    exit(sig);
  }
  printf("... ritorno\n");
  return;
}



void caronte_init() {
  int fd;

  caronte_title(NULL);
  printf( "\n                                Inizializzazione\n\n" );

  // Setup a signals
  if (debug==1) {
    if ( getpid() < 20 ) {
      printf("! turning off debug mode\n");
      debug=0;
    }
  }
  
  // Setup a signals
  printf("- imposto callback segnali\n");
  signal( SIGHUP, caronte_signal );
  signal( SIGBUS, caronte_signal );
  signal( SIGINT, caronte_signal );
  signal( SIGTERM, caronte_signal );
  signal( SIGSEGV, caronte_signal );
  signal( SIGPIPE, caronte_signal );

  // lower kernel verbosity level
  printf("- abbasso il livello di debug del kernel\n");
  klogctl( 8, NULL, 4);

  // disabling CTRL+ALT+DEL
  printf("- disabilita il CTRL+ALT+DEL\n");
  reboot(RB_DISABLE_CAD);

  // Mounting Proc
  if ( 0 != file_exists("/proc/cpuinfo") ) {
    printf("- monta il FS /proc ... ");
    makedir("/proc", 00755 );
    if ( mount( "/proc", "/proc", "proc", 0, NULL ) ) {
      printf("errore: mounting proc\n");
      //caronte_signal(1);
      PRESS_ANY_KEY
    }
    printf("fatto\n");
    sleep(1);
  } else {
    printf("- ottimo, /proc e' gia' montato\n"); 
  }

  // Setting HostName
  printf("- imposto nome host... ");
  sethostname("localhost.localdomain",21);
  setdomainname("",0);
  printf("fatto\n");

#ifdef _AGX_
  // Configurazione Console
  printf("- configuro la console /dev/tty1... ");
  fd = open("/dev/tty1", O_RDWR, 0);
  if (fd < 0) {
    printf("errore: non riesco ad aprire /dev/tty1\n");
    PRESS_ANY_KEY
  } else {
    dup2(fd, 0);
    dup2(fd, 1);
    dup2(fd, 2);
    close(fd);
  }
  // set SID
  setsid();
  if (ioctl(0, TIOCSCTTY, NULL)) {
    printf("errore: impossibile impostare la console di controllo\n");
    PRESS_ANY_KEY
  }
  printf("fatto\n");
#endif

  printf("\n=============================================================================\n");
  printf("ATTENZIONE: Questa non e' la versione definitiva di 'Bad Penguin'.\nQuesta non e' la versione definitiva del programma di setup.\nCi sono e ci potrebbero essere numerosi 'bug', imprecisazioni ed\nimprecisioni. Questo prodotto e' da considerarsi IN SVILUPPO.\nViene fornito cosi' COME E' senza alcuna garanzia implicita o esplicita.\nSiete pregati di effettuare un backup dei vostri dati sensibili prima\ndi continuare la procedura d'installazione.\n\nSei sicuro di voler installare (a tuo rischio e pericolo) Bad Penguin ??\n");
  printf("\n=============================================================================\n");
  
  PRESS_ANY_KEY

}
