#include "config.h"

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/mount.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <badpenguin-ansi.h>
#include <badpenguin-chain.h>
#include <badpenguin-fs.h>
#include <badpenguin-hardware.h>

#include "caronte.h"
#include "caronte-init.h"
#include "caronte-logo.h"
#include "caronte-partitions.h"
#include "caronte-distribution.h"
#include "caronte-mountsource.h"

char sourcedir[80];
char sourceimg[80];

void caronte_mountsource() {
  int iret=0;
  int fd, ffd;
  struct loop_info loopinfo;

  caronte_title(NULL);
  printf( "\n                              Monto distribuzione\n\n" );

  // Make Mount Point and Make sure it is unmounted
  umount(CARONTE_SOURCE_DIR);
  makedir(CARONTE_SOURCE_DIR, S_IRWXU);

  // Monta Distribuzione
  if (0 != strcmp(sourcedist->device,"/")) {
    printf("- monto la distribuzione...\n");
    iret = mount( 
      sourcedist->device, CARONTE_SOURCE_DIR, sourcedist->module, 
      MS_MGC_VAL || MS_RDONLY,
      NULL
    );
    // Errore nel mounting
    if (iret!=0) {
      printf("caronte: impossibile montare la partizione %s (%s) !\n",  sourcedist->device, sourcedist->filesystem );
      PRESS_ANY_KEY
      caronte_reboot();
      exit(1);
    }
    strcpy(sourcedir, CARONTE_SOURCE_DIR);
  } else {
    printf("- utilizzo la distribuzione dal filesystem locale ...\n");
    sourcedir[0] = 0;
  }
  strcat(sourcedir, CARONTE_SOURCE_LOCAL );

  // Monta Immagine
  strcpy(sourceimg, sourcedir);
  strcat(sourceimg, "/stage2.img" );
  if ( 0 != file_exists(sourceimg)){
      printf("caronte: impossibile trovare %s !\n", sourceimg );
      PRESS_ANY_KEY
      caronte_reboot();
      exit(1);  
  }

  // Umount stage2 if any
  umount(CARONTE_SOURCE_STAGE2);
  makedir(CARONTE_SOURCE_STAGE2, S_IRWXU);

  // Set Loopback
  printf("- imposto loopback...\n" );
  ffd = open (sourceimg, O_RDONLY);
  if (ffd<0) {
      printf("caronte: impossibile aprire %s !\n", sourceimg );
      PRESS_ANY_KEY
      caronte_reboot();
      exit(1);  
  }
  
  fd  = open ("/dev/loop0", O_RDONLY);
  if (ffd<0) {
      printf("caronte: impossibile aprire loop device !\n" );
      PRESS_ANY_KEY
      caronte_reboot();
      exit(1);  
  }

  // Clear Loop back if exists (so no error checking required)
  ioctl (fd, LOOP_CLR_FD, ffd);

  memset (&loopinfo, 0, sizeof(loopinfo) );
  strncpy (loopinfo.lo_name, sourceimg, LO_NAME_SIZE);
  loopinfo.lo_name[LO_NAME_SIZE - 1] = 0;
  loopinfo.lo_offset = 0;
  loopinfo.lo_encrypt_key_size = 0;

  if (ioctl (fd, LOOP_SET_FD, ffd) < 0) {
      printf("ioctl: LOOP_SET_FD !\n" );
      PRESS_ANY_KEY
      caronte_reboot();
      exit(1);  
  }

  if (ioctl (fd, LOOP_SET_STATUS, &loopinfo) < 0) {
    (void) ioctl (fd, LOOP_CLR_FD, 0);
    printf("ioctl: LOOP_SET_STATUS !\n" );
    PRESS_ANY_KEY
    caronte_reboot();
    exit(1);  
  }

  close (fd);
  close (ffd);
  
  printf("- monto %s localmente...\n", sourceimg);
  iret = mount( 
      "/dev/loop0", CARONTE_SOURCE_STAGE2, "ext2", 
      MS_MGC_VAL || MS_RDONLY || MS_LOOP,
      NULL
  );
  // Errore nel mounting
  if (iret!=0) {
    printf("caronte: impossibile montare il file %s !\n",  sourceimg );
    PRESS_ANY_KEY
    caronte_reboot();
    exit(1);
  }

}

